Option Explicit

Declare Function CreatePatternBrush Lib "GDI" (ByVal hBitmap As Integer) As Integer
Declare Function Rectangle Lib "GDI" (ByVal hDC As Integer, ByVal LeftX As Integer, ByVal TopY As Integer, ByVal RightX As Integer, ByVal BottomY As Integer) As Integer
Declare Sub InvalidateRect Lib "User" (ByVal hWnd As Integer, ByVal lpRect As Long, ByVal bErase As Integer)

Const NumberOfSteps = 8
Dim PixelSetSequence(64) As Integer
Dim DissolveStep As Integer
Dim DissolveStepForPainting As Integer

Dim bmFileHeader(1) As BITMAPFILEHEADER
Dim bmInfo(1) As BITMAPINFO
Dim hPixelData(1) As Integer
Dim NewImage As Integer

Dim hPalette As Integer
Global palettes(1 To 8) As LOGPALETTE
'Global NewPalette As LOGPALETTE

Type PALSTEP
    Red As Integer
    Green As Integer
    Blue As Integer
    End Type

Dim PalStepArray(255) As PALSTEP

Function CreateDissolveBrush (DissolveStep As Integer, ByVal hDC As Integer) As Integer
    Dim hCompBitmap As Integer
    Dim BrushBitmapInfo As INDEXBITMAPINFO
    Dim Counter As Integer
    Dim PixelData As String * 32
    Dim Dummy As Integer
    Dim Row As Integer
    Dim Column As Integer

    BrushBitmapInfo.bmiHeader.biSize = 40
    BrushBitmapInfo.bmiHeader.biWidth = 8
    BrushBitmapInfo.bmiHeader.biHeight = 8
    BrushBitmapInfo.bmiHeader.biPlanes = 1
    BrushBitmapInfo.bmiHeader.biBitCount = 1
    BrushBitmapInfo.bmiHeader.biCompression = 0
    BrushBitmapInfo.bmiHeader.biSizeImage = 0
    BrushBitmapInfo.bmiHeader.biXPelsPerMeter = 0
    BrushBitmapInfo.bmiHeader.biYPelsPerMeter = 0
    BrushBitmapInfo.bmiHeader.biClrUsed = 0
    BrushBitmapInfo.bmiHeader.biClrImportant = 0

    BrushBitmapInfo.bmiColorIndexes(0) = 0
    BrushBitmapInfo.bmiColorIndexes(1) = 255

    For Counter = 0 To 7
	Mid$(PixelData, Counter * 4 + 1, 1) = Chr$(&HFF)
	Next Counter
    For Counter = 1 To DissolveStep * (64 / NumberOfSteps)
	Row = (PixelSetSequence(Counter) - 1) \ 8
	Column = (PixelSetSequence(Counter) - 1) Mod 8
	Mid$(PixelData, Row * 4 + 1, 1) = Chr$(Asc(Mid$(PixelData, Row * 4 + 1, 1)) And ((Not (2 ^ (7 - Column))) - &HFF00))
	Next Counter

    hCompBitmap = CreateDIBitmapByPal(hDC, BrushBitmapInfo.bmiHeader, CBM_INIT, PixelData, BrushBitmapInfo, DIB_PAL_COLORS)
    CreateDissolveBrush = CreatePatternBrush(hCompBitmap)
    Dummy = DeleteObject%(hCompBitmap)
    End Function

Sub CreatePaletteSequenceArray (CurrentPalette As LOGPALETTE, NewPalette As LOGPALETTE)
    '  This procedure will take the palette belonging to
    '  the currently displayed image, and the palette
    '  belonging to the new image, and will construct a
    '  series of six intermediate palettes.  The eight
    '  palettes will then be used to soften the transition
    '  between images.
    '
    '  The palette array is declared globally.
    
    Dim PalCounter As Integer
    Dim PalEntryCounter As Integer

    If CurrentPalette.palNumEntries <> 256 Then
	LSet palettes(1) = NewPalette
      Else
	LSet palettes(1) = CurrentPalette
      End If
    LSet palettes(8) = NewPalette
    For PalEntryCounter = 0 To 255
	PalStepArray(PalEntryCounter).Red = (Asc(NewPalette.palPalEntry(PalEntryCounter).peRed) - Asc(palettes(1).palPalEntry(PalEntryCounter).peRed)) \ 7
	PalStepArray(PalEntryCounter).Green = (Asc(NewPalette.palPalEntry(PalEntryCounter).peGreen) - Asc(palettes(1).palPalEntry(PalEntryCounter).peGreen)) \ 7
	PalStepArray(PalEntryCounter).Blue = (Asc(NewPalette.palPalEntry(PalEntryCounter).peBlue) - Asc(palettes(1).palPalEntry(PalEntryCounter).peBlue)) \ 7
	Next PalEntryCounter
    For PalCounter = 2 To 7
	For PalEntryCounter = 0 To 255
	    palettes(PalCounter).palPalEntry(PalEntryCounter).peRed = Chr$(MaxInt(MinInt((Asc(palettes(PalCounter - 1).palPalEntry(PalEntryCounter).peRed) + PalStepArray(PalEntryCounter).Red), 255), 0))
	    palettes(PalCounter).palPalEntry(PalEntryCounter).peGreen = Chr$(MaxInt(MinInt((Asc(palettes(PalCounter - 1).palPalEntry(PalEntryCounter).peGreen) + PalStepArray(PalEntryCounter).Green), 255), 0))
	    palettes(PalCounter).palPalEntry(PalEntryCounter).peBlue = Chr$(MaxInt(MinInt((Asc(palettes(PalCounter - 1).palPalEntry(PalEntryCounter).peBlue) + PalStepArray(PalEntryCounter).Blue), 255), 0))
	    If (PalEntryCounter > 9) And (PalEntryCounter < 246) Then
		palettes(PalCounter).palPalEntry(PalEntryCounter).peFlags = Chr$(PC_RESERVED)
	      Else
		palettes(PalCounter).palPalEntry(PalEntryCounter).peFlags = Chr$(0)
	      End If
	    Next PalEntryCounter
	palettes(PalCounter).palVersion = &H300
	palettes(PalCounter).palNumEntries = 256
	Next PalCounter
    End Sub

Sub CreatePixelSetSequence ()
    Dim Counter As Integer
    Dim PixelNumberString As String * 5
    Const PixelListFile = 1

    Open App.Path & "\PixelLst.TXT" For Input As #PixelListFile
    For Counter = 1 To 64
	Input #PixelListFile, PixelNumberString
	PixelSetSequence(Counter) = Val(PixelNumberString)
	Next Counter
    Close
    End Sub

Sub DissolvePaint (DissolvePicture As PictureBox)
    Dim hMemDC As Integer
    Dim hMemBitmap As Integer
    Dim hOldMemBitmap As Integer
    Dim ErrCode As Integer
    Dim lpDIBits As Long
    Dim lpOldDIBits As Long
    Dim hOldPalette As Integer
    Dim hDisolveBrush As Integer
    Dim hOldBrush As Integer
    Dim OldImage As Integer
    Dim OldImageWidth As Integer
    Dim OldImageHeight As Integer
    Dim NewImageWidth As Integer
    Dim NewImageHeight As Integer


    If DissolveStepForPainting < 1 Then Exit Sub
    ' Builds a memory device context where the dissolve
    ' will be performed.
    hMemDC = CreateCompatibleDC(DissolvePicture.hDC)
    hMemBitmap = CreateCompatibleBitmap(DissolvePicture.hDC, DissolvePicture.ScaleWidth, DissolvePicture.ScaleHeight)
    hOldMemBitmap = SelectObject(hMemDC, hMemBitmap)
    ErrCode = Rectangle(hMemDC, 0, 0, DissolvePicture.ScaleWidth, DissolvePicture.ScaleHeight)

    ' Switch the context index.
    OldImage = (NewImage + 1) Mod 2

    ' Assign image dimension info to shorthand reference variables.
    OldImageWidth = bmInfo(OldImage).bmiHeader.biWidth
    OldImageHeight = bmInfo(OldImage).bmiHeader.biHeight
    NewImageWidth = bmInfo(NewImage).bmiHeader.biWidth
    NewImageHeight = bmInfo(NewImage).bmiHeader.biHeight

    ' Select the palette activated by DoDissolveStep() into
    ' the memory and display DCs.
    'ErrCode = UnrealizeObject(hPalette)
    ErrCode = SelectPalette(hMemDC, hPalette, False)
    hOldPalette = SelectPalette(DissolvePicture.hDC, hPalette, False)
    ErrCode = RealizePalette(DissolvePicture.hDC)
    ' Throw away the palette belonging to the previous step or image,
    ' unless this is the first image (you can't delete the
    ' default palette).
    If (hOldPalette <> DEFAULT_PALETTE) And (hOldPalette <> hPalette) Then
	ErrCode = DeleteObject(hOldPalette)
      End If
    ' Lock the memory blocks that contain the pixel data for the
    ' two images.
    lpDIBits = GlobalLock(hPixelData(NewImage))
    If hPixelData(OldImage) <> 0 Then
	lpOldDIBits = GlobalLock(hPixelData(OldImage))
      End If
    If lpDIBits <> 0 Then
	' Create the appropriate dissolve brush for the current step,
	' and select it into the MemDC.
	hDisolveBrush = CreateDissolveBrush(DissolveStepForPainting, DissolvePicture.hDC)
	hOldBrush = SelectObject(hMemDC, hDisolveBrush)
	' Paint the old image into the MemDC with an inverted brush.
	If (hPixelData(OldImage) <> 0) Then
	    ErrCode = StretchDIBits(hMemDC, 0, 0, OldImageWidth, OldImageHeight, 0, 0, OldImageWidth, OldImageHeight, lpOldDIBits, bmInfo(OldImage), DIB_RGB_COLORS, &HCF0224) ' Performs ROP SPno
	    End If
	' Paint the new image into the MemDC with the dissolve ROP.
	ErrCode = StretchDIBits(hMemDC, 0, 0, NewImageWidth, NewImageHeight, 0, 0, NewImageWidth, NewImageHeight, lpDIBits, bmInfo(NewImage), DIB_RGB_COLORS, &HAC0744) ' Performs ROP SPDSxax
	' Rapidly transfer the contents of the MemDC to the screen.
	ErrCode = BitBlt(DissolvePicture.hDC, 0, 0, DissolvePicture.ScaleWidth, DissolvePicture.ScaleHeight, hMemDC, 0, 0, SRCCOPY)
	' Unlock the global memory blocks.
	ErrCode = GlobalUnlock(hPixelData(NewImage))
	If hPixelData(OldImage) <> 0 Then
	    ErrCode = GlobalUnlock(hPixelData(OldImage))
	  End If
	' Replace the default brush in the MemDC and
	' delete the dissolve brush.
	ErrCode = SelectObject(hMemDC, hOldBrush)
	ErrCode = DeleteObject(hDisolveBrush)
      Else
	DissolvePicture.Print "Unable to Set Bits!"
      End If
    ' Delete the MemDC, then delete the MemBitmap.
    ErrCode = SelectObject(hMemDC, hOldMemBitmap)
    ErrCode = SelectPalette(hMemDC, DEFAULT_PALETTE, False)
    ErrCode = DeleteDC(hMemDC)
    ErrCode = DeleteObject(hMemBitmap)

    End Sub

Sub DissolveToImage (FileName As String, DissolveTimer As Timer, DissolveTrigger As Control)
    Dim NewPalette As LOGPALETTE

    DissolveTrigger.Enabled = False
    NewImage = (NewImage + 1) Mod 2
    DissolveStep = 1
    If hPixelData(NewImage) <> 0 Then
	hPixelData(NewImage) = GlobalFree(hPixelData(NewImage))
      End If
    If hPixelData(NewImage) = 0 Then
	ReadBitmapFile FileName, bmFileHeader(NewImage), bmInfo(NewImage), hPixelData(NewImage)
	ConstructPaletteFromColorTable bmInfo(NewImage).bmiColors(), NewPalette
	CreatePaletteSequenceArray palettes(8), NewPalette
	DissolveTimer.Enabled = True
      Else
	MsgBox "Unable To Free Memory from Previous Image", 16, "Memory De-Allocation Error"
      End If
    End Sub

Sub DissolveUnload (DissolvePicture As PictureBox)
    Dim ResultCode As Integer
    If hPixelData(NewImage) <> 0 Then
	ResultCode = GlobalFree(hPixelData(NewImage))
      End If
    If hPixelData((NewImage + 1) Mod 2) <> 0 Then
	ResultCode = GlobalFree(hPixelData((NewImage + 1) Mod 2))
      End If
    ResultCode = SelectPalette(DissolvePicture.hDC, DEFAULT_PALETTE, False)
    If hPalette <> 0 Then
	ResultCode = DeleteObject(hPalette)
      End If

    End Sub

Sub DoDissolveStep (DissolvePicture As PictureBox, DissolveTimer As Timer, DissolveTrigger As Control)
    
    hPalette = CreatePalette(palettes(DissolveStep))
    DissolveStepForPainting = DissolveStep
    InvalidateRect DissolvePicture.hWnd, ByVal 0, False
    'DissolvePicture.Refresh
    If DissolveStep >= 8 Then
	DissolveTimer.Enabled = False
	DissolveTrigger.Enabled = True
      Else
	DissolveStep = DissolveStep + 1
      End If

    End Sub

Sub InitializeDissolve ()
    
    NewImage = 0
    hPixelData(NewImage) = 0
    hPixelData((NewImage + 1) Mod 2) = 0
    hPalette = 0
    CreatePixelSetSequence
    
    End Sub

Function MaxInt (A As Integer, B As Integer)
    If A > B Then
	MaxInt = A
      Else
	MaxInt = B
      End If
      End Function

Function MinInt (A As Integer, B As Integer) As Integer
    If A < B Then
	MinInt = A
      Else
	MinInt = B
      End If
    End Function

