Option Explicit

Type WAVEOUTCAPS
    wMid As Integer
    wPid As Integer
    vDriverVersion As Integer
    szPName As String * 32
    dwFormats As Long
    wChannels As Integer
    dwSupport As Long
    End Type

Type WAVEFORMAT
    wFormatTag As Integer
    nChannels As Integer
    nSamplesPerSec As Long
    nAvgBytesPerSec As Long
    nBlockAlign As Integer
    End Type

Type PCMWAVEFORMAT
    wf As WAVEFORMAT
    wBitsPerSample As Integer
    End Type

Type WAVEHDR
    lpData As Long
    dwBufferLength As Long
    dwBytesRecorded As Long
    dwUser As Long
    dwFlags As Long
    dwLoops As Long
    lpNext As Long
    reserved As Long
    End Type

Type FOURCC
    Chars As String * 4
    End Type

Type MMIOINFO
    dwFlags As Long
    fccIOProc As FOURCC
    lpIOProc As Long
    wErrorRet As Integer
    wReserved As Integer
    ' Fields maintained by MMIO functions during buffered IO
    cchBuffer As Long
    pchBuffer As Long
    pchNext As Long
    pchEndRead As Long
    pchEndWrite As Long
    lBufOffset As Long
    ' Fields maintained by I/O procedure
    lDiskOffset As Long
    adwInfo As String * 12
    ' Other fields maintained by MMIO
    dwReserved1 As Long
    dwReserved2 As Long
    hMMIO As Integer
    End Type

' RIFF chunk information data structure
Type MMCKINFO
    CkId As FOURCC
    CkSize As Long
    fccType As FOURCC
    dwDataOffset As Long
    dwFlags As Long
    End Type

Declare Function waveOutGetDevCaps Lib "MMSystem" (ByVal wDeviceID As Integer, lpCaps As WAVEOUTCAPS, ByVal wSize As Integer) As Integer
Declare Function waveOutOpen Lib "MMSystem" (lphWaveOut As Integer, ByVal wDeviceID As Integer, lpFormat As Any, ByVal dwCallBack As Long, ByVal dwCallBack As Long, ByVal dwFlags As Long) As Integer
Declare Function waveOutClose Lib "MMSystem" (ByVal hWaveOut As Integer) As Integer
Declare Function waveOutPrepareHeader Lib "MMSystem" (ByVal hWaveOut As Integer, lpWaveOutHdr As Any, ByVal wSize As Integer) As Integer
Declare Function waveOutUnprepareHeader Lib "MMSystem" (ByVal hWaveOut As Integer, lpWaveOutHdr As Any, ByVal wSize As Integer) As Integer
Declare Function waveOutWrite Lib "MMSystem" (ByVal hWaveOut As Integer, lpWaveOutHdr As Any, ByVal wSize As Integer) As Integer
Declare Function mmioOpen Lib "MMSystem" (ByVal szFilename As String, lpMMIOINFO As Any, ByVal dwOpenFlags As Long) As Integer
Declare Function mmioClose Lib "MMSystem" (ByVal hMMIO As Integer, ByVal wFlags As Integer) As Integer
Declare Function mmioDescend Lib "MMSystem" (ByVal hMMIO As Integer, lpCk As Any, lpCkParent As Any, ByVal wFlags As Integer) As Integer
Declare Function mmioAscend Lib "MMSystem" (ByVal hMMIO As Integer, lpCk As Any, ByVal wFlags As Integer) As Integer
Declare Function mmioRead Lib "MMSystem" (ByVal hMMIO As Integer, pCh As Any, ByVal cCh As Long) As Long
Declare Function mmioReadToGlobal Lib "MMSystem" Alias "mmioRead" (ByVal hMMIO As Integer, ByVal lpBuffer As Long, ByVal cCh As Long) As Long
Declare Function lstrcpy Lib "Kernel" (lpString1 As Any, lpString2 As Any) As Long

Global Const WAVE_MAPPER = -1        ' Device ID for Wave Mapper
Global Const MMIO_READ = &H0&
Global Const MMIO_WRITE = &H1&
Global Const MMIO_READWRITE = &H2&

Global Const MMIO_FINDCHUNK = &H10   ' mmioDescend: find a chunk by ID
Global Const MMIO_FINDRIFF = &H20    ' mmioDescend: find a LIST chunk

Global Const WHDR_DONE = &H1         ' done bit

' flags for dwFlags parameter in waveOutOpen() and waveInOpen()
Global Const WAVE_FORMAT_QUERY = &H1

Global Const WAVECAPS_PITCH = &H1           ' Supports pitch control
Global Const WAVECAPS_PLAYBACKRATE = &H2    ' Supports playback rate control
Global Const WAVECAPS_VOLUME = &H4          ' Supports volume control
Global Const WAVECAPS_LRVOLUME = &H8        ' Supports separate left-right volume control
Global Const WAVECAPS_SYNC = &H10

Global Const WAVE_INVALIDFORMAT = &H0 ' Invalid Format
Global Const WAVE_FORMAT_1M08 = &H1   ' 11.025 kHz, Mono,   8 bit
Global Const WAVE_FORMAT_1S08 = &H2   ' 11.025 kHz, Stereo, 8 bit
Global Const WAVE_FORMAT_1M16 = &H4   ' 11.025 kHz, Mono,   16 bit
Global Const WAVE_FORMAT_1S16 = &H8   ' 11.025 kHz, Stereo, 16 bit
Global Const WAVE_FORMAT_2M08 = &H10  ' 22.05  kHz, Mono,   8 bit
Global Const WAVE_FORMAT_2S08 = &H20  ' 22.05  kHz, Stereo, 8 bit
Global Const WAVE_FORMAT_2M16 = &H40  ' 22.05  kHz, Mono,   16 bit
Global Const WAVE_FORMAT_2S16 = &H80  ' 22.05  kHz, Stereo, 16 bit
Global Const WAVE_FORMAT_4M08 = &H100 ' 44.1   kHz, Mono,   8 bit
Global Const WAVE_FORMAT_4S08 = &H200 ' 44.1   kHz, Stereo, 8 bit
Global Const WAVE_FORMAT_4M16 = &H400 ' 44.1   kHz, Mono,   16 bit
Global Const WAVE_FORMAT_4S16 = &H800 ' 44.1   kHz, Stereo, 16 bit

Type MonoEightBitSamples
    Char As String * 1
    End Type

Type StereoEightBitSamples
    LeftChar As String * 1
    RightChar As String * 1
    End Type

Type MonoSixteenBitSamples
    Sample As Integer
    End Type

Type StereoSixteenBitSamples
    LeftSample As Integer
    RightSample As Integer
    End Type

Dim hWaveOut As Integer
Dim WaveHeader As WAVEHDR
Global PCMWaveFmtRecord As PCMWAVEFORMAT
Global hWaveSampleData As Integer

Function AddEcho (hTheWaveSampleData As Integer, Delay As Integer, EchoGain As Integer)
    Dim hNewWaveSampleData As Integer
    Dim EchoPeriod As Long
    Dim GainFactor As Single
    Dim lpTheWaveSampleData As Long
    Dim lpNewWaveSampleData As Long
    Dim SelWaveSampleData As Integer
    Dim LastSamplePosition As Long
    Dim Position As Long
    Dim BytesRead As Long
    Dim BytesWritten As Long
    Dim Dummy As Integer

    Dim MonoEightBitSample As MonoEightBitSamples
    Dim PrevMonoEightBitSample As MonoEightBitSamples
    Dim StereoEightBitSample As StereoEightBitSamples
    Dim PrevStereoEightBitSample As StereoEightBitSamples
    Dim MonoSixteenBitSample As MonoSixteenBitSamples
    Dim PrevMonoSixteenBitSample As MonoSixteenBitSamples
    Dim StereoSixteenBitSample As StereoSixteenBitSamples
    Dim PrevStereoSixteenBitSample As StereoSixteenBitSamples
    Const TrailingEchoes = 2
    
    GainFactor = EchoGain / 50
    EchoPeriod = Delay * PCMWaveFmtRecord.wf.nSamplesPerSec \ 1000

    Select Case WaveFormatConstantFromFormat(PCMWaveFmtRecord)
      Case WAVE_FORMAT_1M08, WAVE_FORMAT_2M08, WAVE_FORMAT_4M08
        'MsgBox "Mono 8-bit"
        hTheWaveSampleData = ExtendGlobalMemBlock(hTheWaveSampleData, WaveHeader.dwBufferLength, WaveHeader.dwBufferLength + EchoPeriod * TrailingEchoes)
        SelWaveSampleData = GlobalHandleToSel(hTheWaveSampleData)
        LastSamplePosition = WaveHeader.dwBufferLength + EchoPeriod * TrailingEchoes - 1
        PrevMonoEightBitSample.Char = Chr$(128)
        ' Initialize new bytes to midpoint value.
        For Position = WaveHeader.dwBufferLength To LastSamplePosition
            BytesWritten = MemoryWrite(SelWaveSampleData, Position, PrevMonoEightBitSample, 1)
            Next Position
        ' Mix in echo.
        For Position = EchoPeriod To LastSamplePosition
            BytesRead = MemoryRead(SelWaveSampleData, Position, MonoEightBitSample, 1)
            ' Retrieve contents of byte at (Position-Period).
            BytesRead = MemoryRead(SelWaveSampleData, Position - EchoPeriod, PrevMonoEightBitSample, 1)
            MonoEightBitSample.Char = Chr$(((Asc(MonoEightBitSample.Char) - 128) + (Asc(PrevMonoEightBitSample.Char) - 128) * GainFactor) \ 2 + 128)
            BytesWritten = MemoryWrite(SelWaveSampleData, Position, MonoEightBitSample, 1)
            Next Position
        WaveHeader.dwBufferLength = WaveHeader.dwBufferLength + EchoPeriod * TrailingEchoes
        MsgBox "Completed " + Str$(Position) + " iterations out of " + Str$(WaveHeader.dwBufferLength)
      Case WAVE_FORMAT_1S08, WAVE_FORMAT_2S08, WAVE_FORMAT_4S08
        'MsgBox "Stereo 8-bit"
        EchoPeriod = EchoPeriod * 2
        hTheWaveSampleData = ExtendGlobalMemBlock(hTheWaveSampleData, WaveHeader.dwBufferLength, WaveHeader.dwBufferLength + EchoPeriod * TrailingEchoes)
        SelWaveSampleData = GlobalHandleToSel(hTheWaveSampleData)
        LastSamplePosition = WaveHeader.dwBufferLength + EchoPeriod * TrailingEchoes - 2
        PrevStereoEightBitSample.LeftChar = Chr$(128)
        PrevStereoEightBitSample.RightChar = Chr$(128)
        ' Initialize new bytes to midpoint value.
        For Position = WaveHeader.dwBufferLength To LastSamplePosition Step 2
            BytesWritten = MemoryWrite(SelWaveSampleData, Position, PrevStereoEightBitSample, 2)
            Next Position
        ' Mix in echo.
        For Position = EchoPeriod To LastSamplePosition
            BytesRead = MemoryRead(SelWaveSampleData, Position, StereoEightBitSample, 2)
            ' Retrieve contents of byte at (Position-Period).
            BytesRead = MemoryRead(SelWaveSampleData, Position - EchoPeriod, PrevStereoEightBitSample, 2)
            StereoEightBitSample.LeftChar = Chr$(((Asc(StereoEightBitSample.LeftChar) - 128) + (Asc(PrevStereoEightBitSample.LeftChar) - 128) * GainFactor) \ 2 + 128)
            StereoEightBitSample.RightChar = Chr$(((Asc(StereoEightBitSample.RightChar) - 128) + (Asc(PrevStereoEightBitSample.RightChar) - 128) * GainFactor) \ 2 + 128)
            BytesWritten = MemoryWrite(SelWaveSampleData, Position, StereoEightBitSample, 2)
            Next Position
        WaveHeader.dwBufferLength = WaveHeader.dwBufferLength + EchoPeriod * TrailingEchoes
        MsgBox "Completed " + Str$(Position) + " iterations out of " + Str$(WaveHeader.dwBufferLength)
      Case WAVE_FORMAT_1M16, WAVE_FORMAT_2M16, WAVE_FORMAT_4M16
        'MsgBox "Mono 16-bit"
        EchoPeriod = EchoPeriod * 2
        hTheWaveSampleData = ExtendGlobalMemBlock(hTheWaveSampleData, WaveHeader.dwBufferLength, WaveHeader.dwBufferLength + EchoPeriod * TrailingEchoes)
        SelWaveSampleData = GlobalHandleToSel(hTheWaveSampleData)
        LastSamplePosition = WaveHeader.dwBufferLength + EchoPeriod * TrailingEchoes - 2
        For Position = EchoPeriod To LastSamplePosition Step 2
            BytesRead = MemoryRead(SelWaveSampleData, Position, MonoSixteenBitSample, 2)
            ' Retrieve contents of byte at (Position-Period).
            BytesRead = MemoryRead(SelWaveSampleData, Position - EchoPeriod, PrevMonoSixteenBitSample, 2)
            MonoSixteenBitSample.Sample = (MonoSixteenBitSample.Sample + PrevMonoSixteenBitSample.Sample * GainFactor) \ 2
            BytesWritten = MemoryWrite(SelWaveSampleData, Position, MonoSixteenBitSample, 2)
            Next Position
        WaveHeader.dwBufferLength = WaveHeader.dwBufferLength + EchoPeriod * TrailingEchoes
      Case WAVE_FORMAT_1S16, WAVE_FORMAT_2S16, WAVE_FORMAT_4S16
        'MsgBox "Stereo 16-bit"
        EchoPeriod = EchoPeriod * 4
        hTheWaveSampleData = ExtendGlobalMemBlock(hTheWaveSampleData, WaveHeader.dwBufferLength, WaveHeader.dwBufferLength + EchoPeriod * TrailingEchoes)
        SelWaveSampleData = GlobalHandleToSel(hTheWaveSampleData)
        LastSamplePosition = WaveHeader.dwBufferLength + EchoPeriod * TrailingEchoes - 4
        For Position = EchoPeriod To LastSamplePosition Step 4
            BytesRead = MemoryRead(SelWaveSampleData, Position, StereoSixteenBitSample, 4)
            ' Retrieve contents of byte at (Position-Period).
            BytesRead = MemoryRead(SelWaveSampleData, Position - EchoPeriod, PrevStereoSixteenBitSample, 4)
            StereoSixteenBitSample.LeftSample = (StereoSixteenBitSample.LeftSample + PrevStereoSixteenBitSample.LeftSample * GainFactor) \ 2
            StereoSixteenBitSample.RightSample = (StereoSixteenBitSample.RightSample + PrevStereoSixteenBitSample.RightSample * GainFactor) \ 2
            BytesWritten = MemoryWrite(SelWaveSampleData, Position, StereoSixteenBitSample, 4)
            Next Position
        WaveHeader.dwBufferLength = WaveHeader.dwBufferLength + EchoPeriod * TrailingEchoes
      End Select
    End Function

Sub ChangeLevel (hTheWaveSampleData As Integer, LevelChange As Integer)
    Dim SelWaveSampleData As Integer
    Dim Position As Long
    Dim LevelFactor As Single
    Dim TempValue As Integer
    Dim MonoEightBitSample As MonoEightBitSamples
    Dim StereoEightBitSample As StereoEightBitSamples
    Dim MonoSixteenBitSample As MonoSixteenBitSamples
    Dim StereoSixteenBitSample As StereoSixteenBitSamples
    Dim BytesRead As Long
    Dim BytesWritten As Long
    Dim Dummy As Integer

    SelWaveSampleData = GlobalHandleToSel(hTheWaveSampleData)
    LevelFactor = 1 + LevelChange / 100
    Select Case WaveFormatConstantFromFormat(PCMWaveFmtRecord)
      Case WAVE_FORMAT_1M08, WAVE_FORMAT_2M08, WAVE_FORMAT_4M08
        'MsgBox "Mono 8-bit"
        For Position = 0 To (WaveHeader.dwBufferLength - 1)
            BytesRead = MemoryRead(SelWaveSampleData, Position, MonoEightBitSample, 1)
            TempValue = (Asc(MonoEightBitSample.Char) - 128) * LevelFactor
            MonoEightBitSample.Char = Chr$(MaxInt(MinInt(TempValue, 127), -128) + 128)
            BytesWritten = MemoryWrite(SelWaveSampleData, Position, MonoEightBitSample, 1)
            Next Position
        'MsgBox "Completed " + Str$(Position) + " Iterations out of " + Str$(WaveHeader.dwBufferLength)
      Case WAVE_FORMAT_1S08, WAVE_FORMAT_2S08, WAVE_FORMAT_4S08
        'MsgBox "Stereo 8-bit"
        For Position = 0 To (WaveHeader.dwBufferLength - 2) Step 2
            BytesRead = MemoryRead(SelWaveSampleData, Position, StereoEightBitSample, 2)
            TempValue = (Asc(StereoEightBitSample.LeftChar) - 128) * LevelFactor
            StereoEightBitSample.LeftChar = Chr$(MaxInt(MinInt(TempValue, 127), -128) + 128)
            TempValue = (Asc(StereoEightBitSample.RightChar) - 128) * LevelFactor
            StereoEightBitSample.RightChar = Chr$(MaxInt(MinInt(TempValue, 127), -128) + 128)
            BytesWritten = MemoryWrite(SelWaveSampleData, Position, StereoEightBitSample, 2)
            Next Position
        'MsgBox "Completed " + Str$(Position) + " Iterations out of " + Str$(WaveHeader.dwBufferLength)
      Case WAVE_FORMAT_1M16, WAVE_FORMAT_2M16, WAVE_FORMAT_4M16
        'MsgBox "Mono 16-bit"
        For Position = 0 To (WaveHeader.dwBufferLength - 2) Step 2
            BytesRead = MemoryRead(SelWaveSampleData, Position, MonoSixteenBitSample, 2)
            MonoSixteenBitSample.Sample = MaxInt(MinInt(MonoSixteenBitSample.Sample * LevelFactor, 32767), -32768)
            BytesWritten = MemoryWrite(SelWaveSampleData, Position, MonoSixteenBitSample, 2)
            Next Position
        'MsgBox "Completed " + Str$(Position\2) + " Iterations out of " + Str$(WaveHeader.dwBufferLength\2)
      Case WAVE_FORMAT_1S16, WAVE_FORMAT_2S16, WAVE_FORMAT_4S16
        'MsgBox "Stereo 16-bit"
        For Position = 0 To (WaveHeader.dwBufferLength - 4) Step 4
            BytesRead = MemoryRead(SelWaveSampleData, Position, StereoSixteenBitSample, 4)
            StereoSixteenBitSample.LeftSample = MaxInt(MinInt(StereoSixteenBitSample.LeftSample * LevelFactor, 32767), -32768)
            StereoSixteenBitSample.RightSample = MaxInt(MinInt(StereoSixteenBitSample.RightSample * LevelFactor, 32767), -32768)
            BytesWritten = MemoryWrite(SelWaveSampleData, Position, StereoSixteenBitSample, 4)
            Next Position
        'MsgBox "Completed " + Str$(Position \ 2) + " Iterations out of " + Str$(WaveHeader.dwBufferLength \ 2)
      End Select
    End Sub

Sub CloseWavePlay ()
    Dim Dummy As Integer

    If hWaveSampleData <> 0 Then
        Dummy = GlobalFree(hWaveSampleData)
      End If
    End Sub

Function ExtendGlobalMemBlock (hMemoryBlock As Integer, OldLength As Long, NewLength As Long) As Integer
    Dim hNewMemoryBlock As Integer
    Dim lpNewMemoryBlock As Long
    Dim lpMemoryBlock As Long
    Dim Dummy As Integer

    hNewMemoryBlock = GlobalAlloc(GMEM_MOVEABLE Or GMEM_ZEROINIT, NewLength)
    lpMemoryBlock = GlobalLock(hMemoryBlock)
    lpNewMemoryBlock = GlobalLock(hNewMemoryBlock)
    hmemcpy lpNewMemoryBlock, lpMemoryBlock, OldLength
    Dummy = GlobalUnlock(hMemoryBlock)
    hMemoryBlock = GlobalFree(hMemoryBlock)
    Dummy = GlobalUnlock(hNewMemoryBlock)
    ExtendGlobalMemBlock = hNewMemoryBlock
    End Function

Function OpenWaveFile (ByVal FileNameAndPath As String) As Integer
    Dim MMCKInfoParent As MMCKINFO
    Dim MMCkInfoChild As MMCKINFO
    Dim hMMIO As Integer
    Dim ErrorCode As Integer
    Dim BytesRead As Long
    Dim Index As Integer
    Dim lpWaveSampleData As Long

    hMMIO = mmioOpen(FileNameAndPath, ByVal 0&, MMIO_READ)
    If hMMIO <> 0 Then
        ' Find WAVE Parent Chunk
        MMCKInfoParent.fccType.Chars = "WAVE"
        ErrorCode = mmioDescend(hMMIO, MMCKInfoParent, ByVal 0&, MMIO_FINDRIFF)
        If ErrorCode = 0 Then
            ' Find fmt Chunk
            MMCkInfoChild.CkId.Chars = "fmt "
            ErrorCode = mmioDescend(hMMIO, MMCkInfoChild, MMCKInfoParent, MMIO_FINDCHUNK)
            If ErrorCode = 0 Then
                ' Read PCM Wave Format Record
                BytesRead = mmioRead(hMMIO, PCMWaveFmtRecord, MMCkInfoChild.CkSize)
                If BytesRead > 0 Then
                    ErrorCode = waveOutOpen(hWaveOut, WAVE_MAPPER, PCMWaveFmtRecord, 0&, 0&, WAVE_FORMAT_QUERY)
                    If ErrorCode = 0 Then
                        ' Ascend back one level in the RIFF file.
                        ErrorCode = mmioAscend(hMMIO, MMCkInfoChild, 0)
                        If ErrorCode = 0 Then
                            ' Read data chunk.
                            MMCkInfoChild.CkId.Chars = "data"
                            ErrorCode = mmioDescend(hMMIO, MMCkInfoChild, MMCKInfoParent, MMIO_FINDCHUNK)
                            If ErrorCode = 0 Then
                                hWaveSampleData = GlobalAlloc(GMEM_MOVEABLE Or GMEM_ZEROINIT, MMCkInfoChild.CkSize)
                                If hWaveSampleData <> 0 Then
                                    lpWaveSampleData = GlobalLock(hWaveSampleData)
                                    BytesRead = mmioReadToGlobal(hMMIO, lpWaveSampleData, MMCkInfoChild.CkSize)
                                    lpWaveSampleData = GlobalUnlock(hWaveSampleData)
                                    If BytesRead > 0 Then
                                        WaveHeader.lpData = 0
                                        WaveHeader.dwBufferLength = BytesRead
                                        WaveHeader.dwFlags = 0&
                                        WaveHeader.dwLoops = 0&
                                        OpenWaveFile = True
                                      Else
                                        MsgBox "Couldn't read wave data.", MB_ICONSTOP, "RIFF File Error"
                                      End If
                                  Else
                                    MsgBox "Unable to Allocate Global Memory.", MB_ICONSTOP, "Memory Error"
                                  End If
                              Else
                                MsgBox "Couldn't find data chunk.", MB_ICONSTOP, "RIFF File Error"
                              End If
                          Else
                            MsgBox "Couldn't ascend from fmt chunk.", MB_ICONSTOP, "RIFF File Error"
                          End If
                      Else
                        MsgBox "Format not supported by Wave device.", MB_ICONSTOP, "Wave Data Error"
                      End If
                  Else
                    MsgBox "Couldn't read wave format record.", MB_ICONSTOP, "RIFF File Error"
                  End If
              Else
                MsgBox "Couldn't find fmt chunk.", MB_ICONSTOP, "RIFF File Error"
              End If
          Else
            MsgBox "Couldn't find WAVE parent chunk.", MB_ICONSTOP, "RIFF File Error"
          End If
          ' Close WAVE file.
          ErrorCode = mmioClose(hMMIO, 0)
      Else
        MsgBox "Couldn't open file.", MB_ICONSTOP, "RIFF File Error"
      End If

    End Function

Function WaveFormatConstantFromFormat (ThePCMWaveFormatRecord As PCMWAVEFORMAT) As Long
    Dim SampleRateFactor As Long
    Dim ResolutionFactor As Long
    Dim ChannelsFactor As Long
    
    SampleRateFactor = (Log(ThePCMWaveFormatRecord.wf.nSamplesPerSec \ 11025) / Log(2)) * 4
    ResolutionFactor = (ThePCMWaveFormatRecord.wBitsPerSample \ 8 - 1) * 2
    ChannelsFactor = ThePCMWaveFormatRecord.wf.nChannels - 1
    WaveFormatConstantFromFormat = 2 ^ (SampleRateFactor + ResolutionFactor + ChannelsFactor)
    End Function

Function WaveFormatStringFromConstant (FormatNumber As Long)
    Dim Result As String

    Select Case FormatNumber
      Case WAVE_FORMAT_1M08
          Result = "11.025 kHz, Mono,   8 bit"
      Case WAVE_FORMAT_1S08
          Result = "11.025 kHz, Stereo,  8 bit"
      Case WAVE_FORMAT_1M16
          Result = "11.025 kHz, Mono,   16 bit"
      Case WAVE_FORMAT_1S16
          Result = "11.025 kHz, Stereo,  16 bit"
      Case WAVE_FORMAT_2M08
          Result = "22.05  kHz, Mono,   8 bit"
      Case WAVE_FORMAT_2S08
          Result = "22.05  kHz, Stereo, 8 bit"
      Case WAVE_FORMAT_2M16
          Result = "22.05  kHz, Mono,   16 bit"
      Case WAVE_FORMAT_2S16
          Result = "22.05  kHz, Stereo, 16 bit"
      Case WAVE_FORMAT_4M08
          Result = "44.1   kHz, Mono,   8 bit"
      Case WAVE_FORMAT_4S08
          Result = "44.1   kHz, Stereo, 8 bit"
      Case WAVE_FORMAT_4M16
          Result = "44.1   kHz, Mono,   16 bit"
      Case WAVE_FORMAT_4S16
          Result = "44.1   kHz, Stereo, 16 bit"
      Case Else
          Result = "Invalid Wave Format"
      End Select
    WaveFormatStringFromConstant = Result
    End Function

Function WaveFunctionStringFromConstant (FunctionNumber As Long)
    Dim Result As String

    Select Case FunctionNumber
      Case WAVECAPS_PITCH
          Result = "Pitch Control"
      Case WAVECAPS_PLAYBACKRATE
          Result = "Playback Rate Control"
      Case WAVECAPS_VOLUME
          Result = "Volume Control"
      Case WAVECAPS_LRVOLUME
          Result = "Separate Left-Right Volume Control"
      Case WAVECAPS_SYNC
          Result = "Synchronization"
      Case Else
          Result = "Invalid Function"
      End Select
    WaveFunctionStringFromConstant = Result
    End Function

Function WaveOut () As Integer
    Dim hWaveOut As Integer
    Dim ReturnCode As Integer

    WaveHeader.lpData = GlobalLock(hWaveSampleData)'lstrcpy(WaveBuffer(1), WaveBuffer(1))
    ' Open the wave device.
    ReturnCode = waveOutOpen(hWaveOut, WAVE_MAPPER, PCMWaveFmtRecord, 0&, 0&, 0&)
    If ReturnCode = 0 Then
        ' Prepare the wave output header.
        ReturnCode = waveOutPrepareHeader(hWaveOut, WaveHeader, Len(WaveHeader))
        If ReturnCode = 0 Then
            ' Write the wave data to the output device.
            ReturnCode = waveOutWrite(hWaveOut, WaveHeader, Len(WaveHeader))
            ' Wait until finished playing.
            If ReturnCode = 0 Then
                Do Until (WaveHeader.dwFlags And WHDR_DONE)
                    'DoEvents
                    Loop
              End If
            WaveOut = True
            ' Unprepare the wave output header.
            ReturnCode = waveOutUnprepareHeader(hWaveOut, WaveHeader, Len(WaveHeader))
            If ReturnCode <> 0 Then
                MsgBox "Unable to Unprepare Wave Header", MB_ICONSTOP, "Wave Error"
              End If
            WaveHeader.dwFlags = 0
            ' Close the wave device.
            ReturnCode = waveOutClose(hWaveOut)
            If ReturnCode <> 0 Then
                MsgBox "Unable to Close Wave Device", MB_ICONSTOP, "Wave Error"
              End If
          Else
            ' Couldn't prepare the header, so close the device.
            MsgBox "Unable to Prepare Wave Header", 0, "Wave Error"
            ReturnCode = waveOutClose(hWaveOut)
            If ReturnCode <> 0 Then
                MsgBox "Unable to Close Wave Device", MB_ICONSTOP, "Wave Error"
              End If
          End If
      Else
        ' Couldn't open the device so do nothing.
        MsgBox "Unable to Open Wave Device", MB_ICONSTOP, "Wave Error"
      End If
    WaveHeader.lpData = GlobalUnlock(hWaveSampleData)
    End Function

